@@ -3,18 +3,23 @@ import userEvent from '@testing-library/user-event';
 
 import {
   CoreApp,
+  DataFrame,
+  FieldType,
   getDefaultTimeRange,
   LogLevel,
   LogRowModel,
   LogsDedupStrategy,
   LogsSortOrder,
   store,
+  toDataFrame,
 } from '@grafana/data';
 import { config, reportInteraction } from '@grafana/runtime';
+import { TempoDatasource } from '@grafana-plugins/tempo/datasource';
+import { createTempoDatasource } from '@grafana-plugins/tempo/test/mocks';
 
 import { disablePopoverMenu, enablePopoverMenu, isPopoverMenuDisabled } from '../../utils';
 import { LOG_LINE_BODY_FIELD_NAME } from '../LogDetailsBody';
-import { createLogRow } from '../mocks/logRow';
+import { createLogLine, createLogRow } from '../mocks/logRow';
 import { OTEL_LOG_LINE_ATTRIBUTES_FIELD_NAME, OTEL_PROBE_FIELD } from '../otel/formats';
 
 import { LogList, Props } from './LogList';
@@ -26,21 +31,26 @@ jest.mock('@grafana/assistant', () => ({
   }),
 }));
 
+const FIELDS_LABEL = 'TestLabelType';
+
+const tempoDS: TempoDatasource & {
+  getLabelDisplayTypeFromFrame?: (key: string, frame: DataFrame | undefined, index: number | null) => string | null;
+} = createTempoDatasource(undefined, { uid: 'abc-123' });
+// Test-only override: Mocked data source does not expose getLabelDisplayTypeFromFrame, so we patch it here to show that the viz should work with any data source that returns logs.
+tempoDS.getLabelDisplayTypeFromFrame = () => {
+  return FIELDS_LABEL;
+};
+
 jest.mock('@grafana/runtime', () => {
   return {
     ...jest.requireActual('@grafana/runtime'),
     usePluginLinks: jest.fn().mockReturnValue({ links: [] }),
     reportInteraction: jest.fn(),
-    config: {
-      ...jest.requireActual('@grafana/runtime').config,
-      featureToggles: {
-        ...jest.requireActual('@grafana/runtime').config.featureToggles,
-        logRowsPopoverMenu: true,
-      },
-    },
+    getDataSourceSrv: () => ({
+      get: (uid: string) => Promise.resolve(tempoDS),
+    }),
   };
 });
-
 jest.mock('../../utils', () => ({
   ...jest.requireActual('../../utils'),
   isPopoverMenuDisabled: jest.fn(),
@@ -68,6 +78,7 @@ describe('LogList', () => {
       containerElement: document.createElement('div'),
       dedupStrategy: LogsDedupStrategy.none,
       displayedFields: [],
+      dataFrames: [],
       enableLogDetails: true,
       logs,
       showControls: false,
@@ -101,112 +112,6 @@ describe('LogList', () => {
     expect(onLogRowHover).toHaveBeenCalledWith(expect.objectContaining(logs[0]));
   });
 
-  test('Supports showing log details', async () => {
-    jest.spyOn(store, 'get').mockImplementation((option: string) => {
-      if (option === 'storage-key.detailsMode') {
-        return 'sidebar';
-      }
-      return undefined;
-    });
-    const onClickFilterLabel = jest.fn();
-    const onClickFilterOutLabel = jest.fn();
-    const onClickShowField = jest.fn();
-
-    render(
-      <LogList
-        {...defaultProps}
-        enableLogDetails={true}
-        onClickFilterLabel={onClickFilterLabel}
-        onClickFilterOutLabel={onClickFilterOutLabel}
-        onClickShowField={onClickShowField}
-        logOptionsStorageKey="storage-key"
-      />
-    );
-
-    await userEvent.click(screen.getByText('log message 1'));
-    await screen.findByText('Fields');
-
-    expect(screen.getByText('name_of_the_label')).toBeInTheDocument();
-    expect(screen.getByText('value of the label')).toBeInTheDocument();
-
-    await userEvent.click(screen.getByLabelText('Filter for value in query A'));
-    expect(onClickFilterLabel).toHaveBeenCalledTimes(1);
-
-    await userEvent.click(screen.getByLabelText('Filter out value in query A'));
-    expect(onClickFilterOutLabel).toHaveBeenCalledTimes(1);
-
-    await userEvent.click(screen.getByLabelText('Show this field instead of the message'));
-    expect(onClickShowField).toHaveBeenCalledTimes(1);
-
-    await userEvent.click(screen.getByLabelText('Close log details'));
-
-    expect(screen.queryByText('Fields')).not.toBeInTheDocument();
-    expect(screen.queryByText('Close log details')).not.toBeInTheDocument();
-  });
-
-  test('Supports showing inline log details', async () => {
-    jest.spyOn(store, 'get').mockImplementation((option: string) => {
-      if (option === 'storage-key.detailsMode') {
-        return 'inline';
-      }
-      return undefined;
-    });
-    const onClickFilterLabel = jest.fn();
-    const onClickFilterOutLabel = jest.fn();
-    const onClickShowField = jest.fn();
-
-    render(
-      <LogList
-        {...defaultProps}
-        enableLogDetails={true}
-        onClickFilterLabel={onClickFilterLabel}
-        onClickFilterOutLabel={onClickFilterOutLabel}
-        onClickShowField={onClickShowField}
-        logOptionsStorageKey="storage-key"
-      />
-    );
-
-    await userEvent.click(screen.getByText('log message 1'));
-    await screen.findByText('Fields');
-
-    expect(screen.getByText('name_of_the_label')).toBeInTheDocument();
-    expect(screen.getByText('value of the label')).toBeInTheDocument();
-
-    await userEvent.click(screen.getByLabelText('Filter for value in query A'));
-    expect(onClickFilterLabel).toHaveBeenCalledTimes(1);
-
-    await userEvent.click(screen.getByLabelText('Filter out value in query A'));
-    expect(onClickFilterOutLabel).toHaveBeenCalledTimes(1);
-
-    await userEvent.click(screen.getByLabelText('Show this field instead of the message'));
-    expect(onClickShowField).toHaveBeenCalledTimes(1);
-
-    await userEvent.click(screen.getByLabelText('Close log details'));
-
-    expect(screen.queryByText('Fields')).not.toBeInTheDocument();
-    expect(screen.queryByText('Close log details')).not.toBeInTheDocument();
-  });
-
-  test('Allows people to select text without opening log details', async () => {
-    const spy = jest.spyOn(document, 'getSelection');
-    spy.mockReturnValue({
-      toString: () => 'selected log line',
-      removeAllRanges: () => {},
-      addRange: (range: Range) => {},
-    } as Selection);
-
-    render(<LogList {...defaultProps} enableLogDetails={true} />);
-
-    await userEvent.click(screen.getByText('log message 1'));
-
-    expect(screen.queryByText('name_of_the_label')).not.toBeInTheDocument();
-    expect(screen.queryByText('value of the label')).not.toBeInTheDocument();
-    expect(screen.queryByText('Fields')).not.toBeInTheDocument();
-    expect(screen.queryByText('Close log details')).not.toBeInTheDocument();
-
-    spy.mockRestore();
-  });
-
   test('Shows controls with level filters based on the displayed logs', async () => {
     logs = [createLogRow({ uid: '1', logLevel: LogLevel.info }), createLogRow({ uid: '2', logLevel: LogLevel.debug })];
 
@@ -227,6 +132,57 @@ describe('LogList', () => {
     expect(screen.getByText('debug')).toBeInTheDocument();
   });
 
+  test('Allows to toggle between ms and ns precision timestamps', async () => {
+    logs = [createLogRow({ uid: '1', timeEpochMs: 1754472919504, timeEpochNs: '1754472919504133766' })];
+
+    render(<LogList {...defaultProps} showTime showControls logs={logs} />);
+
+    expect(screen.getByText('2025-08-06 03:35:19.504')).toBeInTheDocument();
+
+    await userEvent.click(screen.getByLabelText('Log timestamps'));
+    await userEvent.click(screen.getByText('Show nanosecond timestamps'));
+
+    expect(screen.getByText('2025-08-06 03:35:19.504133766')).toBeInTheDocument();
+
+    await userEvent.click(screen.getByLabelText('Log timestamps'));
+    await userEvent.click(screen.getByText('Hide timestamps'));
+
+    expect(screen.queryByText(/2025-08-06 03:35:19/)).not.toBeInTheDocument();
+  });
+
+  test('Allows to toggle the display style of logs', async () => {
+    const { container } = render(
+      <LogList
+        {...defaultProps}
+        logs={[logs[0]]}
+        displayedFields={['name_of_the_label', LOG_LINE_BODY_FIELD_NAME, 'empty_field_without_value']}
+        showTime
+        showControls
+        wrapLogMessage
+        unwrappedColumns={false}
+      />
+    );
+
+    // 3 displayed fields - 1 empty field + time + level
+    expect(container.querySelectorAll('.field')).toHaveLength(4);
+    expect(screen.queryByLabelText('Enable columns')).not.toBeInTheDocument();
+    expect(screen.queryByLabelText('Disable columns')).not.toBeInTheDocument();
+
+    await userEvent.click(screen.getByLabelText('Set line wrap'));
+    await userEvent.click(screen.getByText('Disable line wrapping'));
+
+    // 3 displayed fields - 1 empty field + time + level
+    expect(container.querySelectorAll('.field')).toHaveLength(4);
+    expect(screen.getByLabelText('Enable columns')).toBeInTheDocument();
+
+    await userEvent.click(screen.getByLabelText('Enable columns'));
+
+    // 3 displayed fields + time + level
+    expect(container.querySelectorAll('.field')).toHaveLength(5);
+    expect(screen.getByLabelText('Disable columns')).toBeInTheDocument();
+    expect(screen.queryByLabelText('Enable columns')).not.toBeInTheDocument();
+  });
+
   describe('OTel log lines', () => {
     const originalState = config.featureToggles.otelLogsFormatting;
 
@@ -386,7 +342,7 @@ describe('LogList', () => {
         });
         await userEvent.click(screen.getByText('log message 1'));
         expect(screen.queryByText('Copy selection')).not.toBeInTheDocument();
-        expect(screen.getByText(/Fields/)).toBeInTheDocument();
+        expect(screen.getByText(FIELDS_LABEL)).toBeInTheDocument();
       });
     });
   });
@@ -445,24 +401,6 @@ describe('LogList', () => {
       expect(screen.getByText('log message 1')).toBeInTheDocument();
       expect(screen.getByText('some text')).toBeInTheDocument();
     });
-
-    test('Allows to toggle between ms and ns precision timestamps', async () => {
-      logs = [createLogRow({ uid: '1', timeEpochMs: 1754472919504, timeEpochNs: '1754472919504133766' })];
-
-      render(<LogList {...defaultProps} showTime showControls logs={logs} />);
-
-      expect(screen.getByText('2025-08-06 03:35:19.504')).toBeInTheDocument();
-
-      await userEvent.click(screen.getByLabelText('Log timestamps'));
-      await userEvent.click(screen.getByText('Show nanosecond timestamps'));
-
-      expect(screen.getByText('2025-08-06 03:35:19.504133766')).toBeInTheDocument();
-
-      await userEvent.click(screen.getByLabelText('Log timestamps'));
-      await userEvent.click(screen.getByText('Hide timestamps'));
-
-      expect(screen.queryByText(/2025-08-06 03:35:19/)).not.toBeInTheDocument();
-    });
   });
   describe('Interactions', () => {
     beforeEach(() => {
@@ -480,4 +418,288 @@ describe('LogList', () => {
       expect(reportInteraction).not.toHaveBeenCalled();
     });
   });
+  describe('Field selector', () => {
+    const logs = [
+      createLogLine({ uid: '1', entry: 'log 1', labels: { service: 'frontend', level: 'info' } }),
+      createLogLine({ uid: '2', entry: 'log 2', labels: { service: 'backend', level: 'error' } }),
+    ];
+
+    const dataFrames = [
+      toDataFrame({
+        fields: [
+          { name: 'timestamp', type: FieldType.time, values: [1, 2] },
+          { name: 'body', type: FieldType.string, values: ['log 1', 'log 2'] },
+          {
+            name: 'labels',
+            type: FieldType.other,
+            values: [
+              { service: 'frontend', level: 'info' },
+              { service: 'backend', level: 'error' },
+            ],
+          },
+        ],
+      }),
+    ];
+
+    const extraProps = {
+      onClickShowField: jest.fn(),
+      onClickHideField: jest.fn(),
+      setDisplayedFields: jest.fn(),
+      logs,
+      dataFrames,
+    };
+
+    test('Does not display the field selector if not enabled', async () => {
+      render(<LogList {...defaultProps} {...extraProps} showFieldSelector={false} />);
+
+      expect(screen.queryByPlaceholderText('Search fields by name')).not.toBeInTheDocument();
+      expect(screen.queryByText('Fields')).not.toBeInTheDocument();
+    });
+
+    test('Displays the field selector if enabled', async () => {
+      render(<LogList {...defaultProps} {...extraProps} showFieldSelector />);
+
+      expect(screen.getByPlaceholderText('Search fields by name')).toBeInTheDocument();
+      expect(screen.getByText('Fields')).toBeInTheDocument();
+    });
+
+    test('Toggles displayed fields on and off', async () => {
+      const { rerender } = render(<LogList {...defaultProps} {...extraProps} showFieldSelector />);
+
+      await screen.findByText('log 1');
+      await screen.findByText('log 2');
+
+      await userEvent.click(screen.getByText('service'));
+
+      expect(extraProps.onClickShowField).toHaveBeenCalledWith('service');
+
+      rerender(<LogList {...defaultProps} displayedFields={['service']} {...extraProps} showFieldSelector />);
+
+      expect(screen.getByText('frontend')).toBeInTheDocument();
+      expect(screen.getByText('backend')).toBeInTheDocument();
+
+      await userEvent.click(screen.getByText('service'));
+
+      expect(extraProps.onClickHideField).toHaveBeenCalledWith('service');
+    });
+
+    test('Applies OTel default displayed fields and suggested fields', () => {
+      const originalState = config.featureToggles.otelLogsFormatting;
+      config.featureToggles.otelLogsFormatting = true;
+
+      const logs = [
+        createLogRow({
+          uid: '1',
+          entry: 'log message 1',
+          labels: { service: 'frontend', level: 'info', [OTEL_PROBE_FIELD]: '1', scope_name: 'test' },
+        }),
+      ];
+
+      render(<LogList {...defaultProps} {...extraProps} showFieldSelector logs={logs} />);
+
+      // Log line message
+      expect(screen.getByText('log message 1')).toBeInTheDocument();
+
+      // Label
+      expect(screen.getByText('service')).toBeInTheDocument();
+
+      // Default displayed fields
+      expect(screen.getByText('Log line')).toBeInTheDocument();
+      expect(screen.getByText('Log attributes')).toBeInTheDocument();
+
+      // Suggested field
+      expect(screen.getByText('scope_name')).toBeInTheDocument();
+
+      config.featureToggles.otelLogsFormatting = originalState;
+    });
+  });
+
+  describe('Log details', () => {
+    test('Supports showing log details', async () => {
+      jest.spyOn(store, 'get').mockImplementation((option: string) => {
+        if (option === 'storage-key.detailsMode') {
+          return 'sidebar';
+        }
+        return undefined;
+      });
+      const onClickFilterLabel = jest.fn();
+      const onClickFilterOutLabel = jest.fn();
+      const onClickShowField = jest.fn();
+
+      render(
+        <LogList
+          {...defaultProps}
+          enableLogDetails={true}
+          onClickFilterLabel={onClickFilterLabel}
+          onClickFilterOutLabel={onClickFilterOutLabel}
+          onClickShowField={onClickShowField}
+          logOptionsStorageKey="storage-key"
+        />
+      );
+
+      await userEvent.click(screen.getByText('log message 1'));
+      await screen.findByText(FIELDS_LABEL);
+
+      expect(screen.getByText('name_of_the_label')).toBeInTheDocument();
+      expect(screen.getByText('value of the label')).toBeInTheDocument();
+
+      await userEvent.click(screen.getByLabelText('Filter for value in query A'));
+      expect(onClickFilterLabel).toHaveBeenCalledTimes(1);
+
+      await userEvent.click(screen.getByLabelText('Filter out value in query A'));
+      expect(onClickFilterOutLabel).toHaveBeenCalledTimes(1);
+
+      await userEvent.click(screen.getByLabelText('Show this field instead of the message'));
+      expect(onClickShowField).toHaveBeenCalledTimes(1);
+
+      await userEvent.click(screen.getByLabelText('Close log details'));
+
+      expect(screen.queryByText('Fields')).not.toBeInTheDocument();
+      expect(screen.queryByText('Close log details')).not.toBeInTheDocument();
+    });
+
+    test('Supports showing inline log details', async () => {
+      jest.spyOn(store, 'get').mockImplementation((option: string) => {
+        if (option === 'storage-key.detailsMode') {
+          return 'inline';
+        }
+        return undefined;
+      });
+      const onClickFilterLabel = jest.fn();
+      const onClickFilterOutLabel = jest.fn();
+      const onClickShowField = jest.fn();
+
+      render(
+        <LogList
+          {...defaultProps}
+          enableLogDetails={true}
+          onClickFilterLabel={onClickFilterLabel}
+          onClickFilterOutLabel={onClickFilterOutLabel}
+          onClickShowField={onClickShowField}
+          logOptionsStorageKey="storage-key"
+        />
+      );
+
+      await userEvent.click(screen.getByText('log message 1'));
+      await screen.findByText(FIELDS_LABEL);
+
+      expect(screen.getByText('name_of_the_label')).toBeInTheDocument();
+      expect(screen.getByText('value of the label')).toBeInTheDocument();
+
+      await userEvent.click(screen.getByLabelText('Filter for value in query A'));
+      expect(onClickFilterLabel).toHaveBeenCalledTimes(1);
+
+      await userEvent.click(screen.getByLabelText('Filter out value in query A'));
+      expect(onClickFilterOutLabel).toHaveBeenCalledTimes(1);
+
+      await userEvent.click(screen.getByLabelText('Show this field instead of the message'));
+      expect(onClickShowField).toHaveBeenCalledTimes(1);
+
+      await userEvent.click(screen.getByLabelText('Close log details'));
+
+      expect(screen.queryByText('Fields')).not.toBeInTheDocument();
+      expect(screen.queryByText('Close log details')).not.toBeInTheDocument();
+    });
+
+    test('Allows people to select text without opening log details', async () => {
+      const spy = jest.spyOn(document, 'getSelection');
+      spy.mockReturnValue({
+        toString: () => 'selected log line',
+        removeAllRanges: () => {},
+        addRange: (range: Range) => {},
+      } as Selection);
+
+      render(<LogList {...defaultProps} enableLogDetails={true} />);
+
+      await userEvent.click(screen.getByText('log message 1'));
+
+      expect(screen.queryByText('name_of_the_label')).not.toBeInTheDocument();
+      expect(screen.queryByText('value of the label')).not.toBeInTheDocument();
+      expect(screen.queryByText('Fields')).not.toBeInTheDocument();
+      expect(screen.queryByText('Close log details')).not.toBeInTheDocument();
+
+      spy.mockRestore();
+    });
+
+    test('Renders multiple log details', async () => {
+      const logs = [
+        createLogLine({ uid: '1', logLevel: LogLevel.error, timeEpochMs: 1546297200000, entry: 'First log' }),
+        createLogLine({ uid: '2', logLevel: LogLevel.error, timeEpochMs: 1546297200000, entry: 'Second log' }),
+      ];
+
+      render(<LogList {...defaultProps} enableLogDetails={true} logs={logs} detailsMode="sidebar" />);
+
+      // Open details of 2 logs
+      await userEvent.click(screen.getByText('First log'));
+      await userEvent.click(screen.getByText('Second log'));
+
+      // 2 tabs
+      expect(screen.queryAllByRole('tab')).toHaveLength(2);
+
+      // Expand Log line section inside Details
+      await userEvent.click(screen.getByText('Log line'));
+
+      // Tab + log line in the list
+      expect(screen.getAllByText('First log')).toHaveLength(2);
+      // Tab + log line in the list + Log details (active tab)
+      expect(screen.getAllByText('Second log')).toHaveLength(3);
+
+      // Make first log active
+      await userEvent.click(screen.queryAllByRole('tab')[1]);
+
+      // Tab + log line in the list + Log details (active tab)
+      expect(screen.getAllByText('First log')).toHaveLength(3);
+      // Tab + log line in the list
+      expect(screen.getAllByText('Second log')).toHaveLength(2);
+    });
+
+    test('Changes details focus when logs are added and removed', async () => {
+      const logs = [
+        createLogLine({ uid: '1', logLevel: LogLevel.error, timeEpochMs: 1546297200000, entry: 'First log' }),
+        createLogLine({ uid: '2', logLevel: LogLevel.error, timeEpochMs: 1546297200000, entry: 'Second log' }),
+        createLogLine({ uid: '3', logLevel: LogLevel.error, timeEpochMs: 1546297200000, entry: 'Third log' }),
+      ];
+
+      render(<LogList {...defaultProps} enableLogDetails={true} logs={logs} detailsMode="sidebar" />);
+
+      // No details shown
+      expect(screen.queryByPlaceholderText('Search field names and values')).not.toBeInTheDocument();
+
+      await userEvent.click(screen.getByText('First log'));
+
+      // Details shown
+      expect(screen.getByPlaceholderText('Search field names and values')).toBeInTheDocument();
+
+      // No tabs, only one details displayed
+      expect(screen.queryAllByRole('tab')).toHaveLength(0);
+
+      await userEvent.click(screen.getByText('Second log'));
+
+      // 2 details displayed, Second log is the first tab
+      expect(screen.queryAllByRole('tab')).toHaveLength(2);
+      expect(screen.queryAllByRole('tab')[0]).toHaveTextContent('Second log');
+
+      await userEvent.click(screen.getByText('Third log'));
+
+      // 3 details displayed, Second log is the first tab
+      expect(screen.queryAllByRole('tab')).toHaveLength(3);
+      expect(screen.queryAllByRole('tab')[0]).toHaveTextContent('Third log');
+
+      await userEvent.click(screen.getAllByText('Third log')[1]);
+
+      // 2 details displayed, Second log is the first tab
+      expect(screen.queryAllByRole('tab')).toHaveLength(2);
+      expect(screen.queryAllByRole('tab')[0]).toHaveTextContent('Second log');
+
+      await userEvent.click(screen.getAllByText('Second log')[1]);
+
+      // No tabs, only one details displayed
+      expect(screen.queryAllByRole('tab')).toHaveLength(0);
+
+      await userEvent.click(screen.getByText('First log'));
+
+      // No details shown
+      expect(screen.queryByPlaceholderText('Search field names and values')).not.toBeInTheDocument();
+    });
+  });
 });
